/*
 *  SoapUI IntelliJ Plugin, copyright (C) 2006 eviware.com
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package com.eviware.soapui.intellij.plugin;

import java.awt.*;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jetbrains.annotations.NotNull;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.SwingSoapUICore;
import com.eviware.soapui.SoapUICore;
import com.eviware.soapui.SwingPluginSoapUICore;
import com.eviware.soapui.actions.SoapUIPreferencesAction;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.model.util.PanelBuilderRegistry;
import com.eviware.soapui.model.workspace.Workspace;
import com.eviware.soapui.model.workspace.WorkspaceFactory;
import com.eviware.soapui.monitor.TestMonitor;
import com.eviware.soapui.monitor.support.TestMonitorListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.swing.SwingActionDelegate;
import com.eviware.soapui.support.log.TabbedLog4JMonitor;
import com.eviware.soapui.ui.Navigator;
import com.eviware.soapui.ui.NavigatorListener;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.desktop.SoapUIDesktop;
import com.eviware.soapui.ui.support.DesktopListenerAdapter;
import com.eviware.x.impl.swing.SwingDialogs;
import com.eviware.x.impl.swing.SwingFileDialogs;
import com.eviware.x.dialogs.XDialogs;
import com.eviware.x.dialogs.XFileDialogs;
import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.openapi.wm.WindowManager;

/**
 * Main Plugin class, sets up SoapUI core and creates toolwindows
 */

public class SoapUIPlugin implements ProjectComponent
{
	private Project myProject;

	private static Logger log;
	private static final String WORKSPACE_WINDOW_ID = "soapUI Navigator";
	private Navigator navigator;
	private static Workspace workspace;
	private static final String LOG_WINDOW_ID = "soapUI Log";
	private JPanel overviewPanel;
	private static final String DEFAULT_WORKSPACE_FILE = "intellij-soapui-workspace.xml";
	private static SwingSoapUICore soapUICore;
    private SwingDialogs swingDialogs;
    private SwingFileDialogs swingFileDialogs;
    private Window window;
    private IntellijSoapUIDesktop desktop;
    private JPanel logPanel;
    private SoapUIPlugin.InternalDesktopListenerAdapter internalDesktopListenerAdapter;

    public SoapUIPlugin( Project project )
	{
		myProject = project;
    }

	public void projectOpened()
	{
		initToolWindow();
        window = WindowManager.getInstance().suggestParentWindow( myProject );
        UISupport.setMainFrame(window);
        swingDialogs = new SwingDialogs( UISupport.getMainFrame() );
        UISupport.setDialogs(swingDialogs);
        swingFileDialogs = new SwingFileDialogs( UISupport.getMainFrame() );
        UISupport.setFileDialogs(swingFileDialogs);

        WindowManager.getInstance().getFrame( myProject ).addWindowFocusListener(new WindowAdapter()
        {
            public void windowGainedFocus(WindowEvent windowEvent) {
                SoapUI.setNavigator( navigator );
			    SoapUI.setDesktop( desktop );
                UISupport.setMainFrame(window);
                UISupport.setDialogs(swingDialogs);
                UISupport.setFileDialogs(swingFileDialogs);
                logPanel.add( SoapUI.getLogMonitor().getComponent(), BorderLayout.CENTER );
            }
        });
    }

    public void projectClosed()
	{
		unregisterToolWindow();

        navigator = null;
        desktop.removeDesktopListener(internalDesktopListenerAdapter);
        desktop.release();
    }

	public void initComponent()
	{
		ClassLoader loader = Thread.currentThread().getContextClassLoader();
		Thread.currentThread().setContextClassLoader( SoapUI.class.getClassLoader() );

		try
		{
			if( soapUICore == null )
			{
				soapUICore = new SwingPluginSoapUICore();

				SoapUI.initLogMonitor( true, "soapui", new TabbedLog4JMonitor() );
				new SoapUIPreferencesAction();

				log = Logger.getLogger( SoapUIPlugin.class );
				SwingActionDelegate.switchClassloader = true;
			}

            if( workspace == null )
            {
                workspace = WorkspaceFactory.getInstance().openWorkspace(
						new String[] { System.getProperty( "user.home" ) + File.separatorChar + DEFAULT_WORKSPACE_FILE } );

			    soapUICore.afterStartup( workspace );

                TestMonitor testMonitor = SoapUI.getTestMonitor();

			    testMonitor.addTestMonitorListener( new InternalTestMonitorListener() );
			    testMonitor.init( workspace );
            }

            navigator = new Navigator( workspace );
			navigator.addNavigatorListener( new InternalNavigatorListener() );

            desktop = new IntellijSoapUIDesktop( workspace, myProject );
            internalDesktopListenerAdapter = new InternalDesktopListenerAdapter();
            desktop.addDesktopListener(internalDesktopListenerAdapter);

			SoapUI.setNavigator( navigator );
			SoapUI.setDesktop( desktop );
		}
		catch( Exception e )
		{
			SoapUI.logError( e );
		}
		finally
		{
			Thread.currentThread().setContextClassLoader( loader );
		}
	}

	public void disposeComponent()
	{
		if( workspace != null )
		{
			workspace.save( false );
			try
			{
				SoapUI.saveSettings();
			}
			catch( Exception e )
			{
				SoapUI.logError( e );
			}
		}
	}

	@NotNull
	public String getComponentName()
	{
		return "SoapUI Plugin";
	}

	private void initToolWindow()
	{
		ToolWindowManager toolWindowManager = ToolWindowManager.getInstance( myProject );

		toolWindowManager.registerToolWindow( WORKSPACE_WINDOW_ID, buildMainPanel(), ToolWindowAnchor.LEFT );

        logPanel = new JPanel( new BorderLayout() );
        logPanel.add( SoapUI.getLogMonitor().getComponent(), BorderLayout.CENTER );

        toolWindowManager.registerToolWindow( LOG_WINDOW_ID, logPanel, ToolWindowAnchor.BOTTOM );
	}

	private void unregisterToolWindow()
	{
		ToolWindowManager toolWindowManager = ToolWindowManager.getInstance( myProject );
		toolWindowManager.unregisterToolWindow( WORKSPACE_WINDOW_ID );
		toolWindowManager.unregisterToolWindow( LOG_WINDOW_ID );
	}

	private JComponent buildMainPanel()
	{
		JSplitPane splitPane = UISupport.createVerticalSplit();
      
        splitPane.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );

		splitPane.setTopComponent( navigator );
		splitPane.setBottomComponent( buildOverviewPanel() );
		splitPane.setDividerLocation( 0.7 );
		splitPane.setResizeWeight( 0.7 );
		return splitPane;
	}

	private Component buildOverviewPanel()
	{
		overviewPanel = new JPanel( new BorderLayout() );
		return overviewPanel;
	}

	private void setOverviewPanel( Component panel )
	{
		if( overviewPanel.getComponentCount() == 0 && panel == null )
		{
			return;
		}

		overviewPanel.removeAll();
		if( panel != null )
		{
			overviewPanel.add( panel, BorderLayout.CENTER );
		}

		overviewPanel.revalidate();
		overviewPanel.repaint();
	}

	public class InternalNavigatorListener implements NavigatorListener
	{
		public void nodeSelected( SoapUITreeNode treeNode )
		{
			if( treeNode == null )
			{
				setOverviewPanel( null );
			}
			else
			{
				PanelBuilder<ModelItem> panelBuilder = PanelBuilderRegistry.getPanelBuilder( treeNode.getModelItem() );
				if( panelBuilder != null && panelBuilder.hasOverviewPanel() )
				{
					setOverviewPanel( panelBuilder.buildOverviewPanel( treeNode.getModelItem() ) );
				}
				else
				{
					setOverviewPanel( null );
				}
			}
		}
	}

	private static final class InternalTestMonitorListener extends TestMonitorListenerAdapter
	{
		private Set<LoadTestRunner> loadTestRunners = new HashSet<LoadTestRunner>();

		public void loadTestStarted( LoadTestRunner runner )
		{
			if( loadTestRunners.isEmpty() )
			{
				log.info( "Disabling httpclient and groovy logs during loadtests" );
				Logger.getLogger( "httpclient.wire" ).setLevel( Level.OFF );
				Logger.getLogger( "groovy.log" ).setLevel( Level.OFF );
			}

			loadTestRunners.add( runner );
		}

		public void loadTestFinished( LoadTestRunner runner )
		{
			loadTestRunners.remove( runner );

			if( loadTestRunners.isEmpty() )
			{
				Logger.getLogger( "httpclient.wire" ).setLevel( Level.DEBUG );
				Logger.getLogger( "groovy.log" ).setLevel( Level.DEBUG );
				log.info( "Enabled httpclient and groovy logs after loadtests" );
			}
		}
	}

    private class InternalDesktopListenerAdapter extends DesktopListenerAdapter {
        public void desktopPanelSelected( DesktopPanel desktopPanel )
        {
            ModelItem modelItem = desktopPanel.getModelItem();
            if( modelItem != null && navigator != null )
            {
                navigator.selectModelItem( modelItem );
            }
        }
    }
}
